==> kryptokrona-node-0.2.0-sources.jar: org/kryptokrona/sdk/node/client/NodeClient.kt <==
// Copyright (c) 2022-2023, The Kryptokrona Developers
//
// Written by Marcus Cvjeticanin
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
//    conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
//    of conditions and the following disclaimer in the documentation and/or other
//    materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors may be
//    used to endorse or promote products derived from this software without specific
//    prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package org.kryptokrona.sdk.node.client

import io.ktor.client.call.*
import io.ktor.client.plugins.*
import io.ktor.client.request.*
import io.ktor.http.*
import io.ktor.serialization.*
import org.kryptokrona.sdk.node.common.HttpClient.client
import org.kryptokrona.sdk.node.model.response.node.FeeResponse
import org.kryptokrona.sdk.node.model.response.node.HeightResponse
import org.kryptokrona.sdk.node.model.response.node.InfoResponse
import org.kryptokrona.sdk.node.model.response.node.PeersResponse
import org.kryptokrona.sdk.util.model.node.Node
import org.slf4j.LoggerFactory
import java.nio.channels.UnresolvedAddressException

/**
 * Node client
 *
 * @author Marcus Cvjeticanin
 * @since 0.1.0
 * @param node The node that the wallet service is connected to.
 */
class NodeClient(private val node: Node) {

    private val logger = LoggerFactory.getLogger("NodeClient")

    /**
     * Check if node is running
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return Boolean
     */
    suspend fun isNodeRunning(): Boolean {
        var isSuccess = false

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/info"
                val response = client.get(url)
                isSuccess = response.status.isSuccess()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node information. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node information. Could not resolve the address.", e)
        }

        return isSuccess
    }

    /**
     * Get node info
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return InfoResponse
     */
    suspend fun getNodeInfo(): InfoResponse? {
        var result: InfoResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/info"
                result = client.get(url).body<InfoResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node information. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node information. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node information. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node height
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return HeightResponse
     */
    suspend fun getNodeHeight(): HeightResponse? {
        var result: HeightResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/height"
                result = client.get(url).body<HeightResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node height. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node height. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node height. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node peers
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return PeersResponse
     */
    suspend fun getNodePeers(): PeersResponse? {
        var result: PeersResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/peers"
                result = client.get(url).body<PeersResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node peers. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node peers. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node peers. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node fee
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return FeeResponse
     */
    suspend fun getNodeFee(): FeeResponse? {
        var result: FeeResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/fee"
                result = client.get(url).body<FeeResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node fee. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node fee. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node fee. Could not parse the response.", e)
        }

        return result
    }
}

==> kryptokrona-node-0.3.0-sources.jar: org/kryptokrona/sdk/node/client/NodeClient.kt <==
// Copyright (c) 2022-2023, The Kryptokrona Developers
//
// Written by Marcus Cvjeticanin
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this list of
//    conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice, this list
//    of conditions and the following disclaimer in the documentation and/or other
//    materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors may be
//    used to endorse or promote products derived from this software without specific
//    prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
// THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
// THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package org.kryptokrona.sdk.node.client

import io.ktor.client.call.*
import io.ktor.client.network.sockets.*
import io.ktor.client.plugins.*
import io.ktor.client.request.*
import io.ktor.http.*
import io.ktor.serialization.*
import org.kryptokrona.sdk.node.common.HttpClient.client
import org.kryptokrona.sdk.node.model.response.node.FeeResponse
import org.kryptokrona.sdk.node.model.response.node.HeightResponse
import org.kryptokrona.sdk.node.model.response.node.InfoResponse
import org.kryptokrona.sdk.node.model.response.node.PeersResponse
import org.kryptokrona.sdk.util.model.node.Node
import org.slf4j.LoggerFactory
import java.nio.channels.UnresolvedAddressException

/**
 * Node client
 *
 * @author Marcus Cvjeticanin
 * @since 0.1.0
 * @param node The node that the wallet service is connected to.
 */
class NodeClient(private val node: Node) {

    private val logger = LoggerFactory.getLogger("NodeClient")

    /**
     * Check if node is running
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return Boolean
     */
    suspend fun isNodeRunning(): Boolean {
        var isSuccess = false

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/info"
                val response = client.get(url)
                isSuccess = response.status.isSuccess()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node information. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node information. Could not resolve the address.", e)
        } catch (e: ConnectTimeoutException) {
            logger.error("Error getting node information. Connection timeout.", e)
        }

        return isSuccess
    }

    /**
     * Get node info
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return InfoResponse
     */
    suspend fun getNodeInfo(): InfoResponse? {
        var result: InfoResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/info"
                result = client.get(url).body<InfoResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node information. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node information. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node information. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node height
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return HeightResponse
     */
    suspend fun getNodeHeight(): HeightResponse? {
        var result: HeightResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/height"
                result = client.get(url).body<HeightResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node height. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node height. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node height. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node peers
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return PeersResponse
     */
    suspend fun getNodePeers(): PeersResponse? {
        var result: PeersResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/peers"
                result = client.get(url).body<PeersResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node peers. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node peers. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node peers. Could not parse the response.", e)
        }

        return result
    }

    /**
     * Get node fee
     *
     * @author Marcus Cvjeticanin
     * @since 0.1.0
     * @return FeeResponse
     */
    suspend fun getNodeFee(): FeeResponse? {
        var result: FeeResponse? = null

        try {
            node.ssl.let {
                val protocol = if (it) "https" else "http"
                val url = "$protocol://${node.hostName}:${node.port}/fee"
                result = client.get(url).body<FeeResponse>()
            }
        } catch (e: HttpRequestTimeoutException) {
            logger.error("Error getting node fee. Could not reach the server.", e)
        } catch (e: UnresolvedAddressException) {
            logger.error("Error getting node fee. Could not resolve the address.", e)
        } catch (e: JsonConvertException) {
            logger.error("Error getting node fee. Could not parse the response.", e)
        }

        return result
    }
}
